﻿using System;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Shapes;

namespace Effects2D.Brushes
{
    /// <summary>
    /// Interaction logic for EffectLinearGradientBrush.xaml
    /// </summary>
    public partial class EffectLinearGradientBrush : UserControl
    {
        private bool captured = false;
        private double x_canvasS, y_canvasS, x_canvasE, y_canvasE;
        private double xStart, yStart, xEnd, yEnd;
        private UIElement pointStart = null;
        private UIElement pointEnd = null;
        private Point startPoint;
        private Point endPoint;
        private Line line = new Line();
        private double radius = 0;
        private double minX, maxX, minY, maxY;
        private double minXframe, maxXframe, minYframe, maxYframe;

        public EffectLinearGradientBrush()
        {
            InitializeComponent();

            radius = start.Width / 2;

            xStart = (rctBase.Width / 2);
            yStart = 0;
            xEnd = (rctBase.Width / 2);
            yEnd = rctBase.Height;

            minX = 0;
            maxX = rctBase.Width;
            minY = 0;
            maxY = rctBase.Height;

            minXframe = -radius;
            maxXframe = rctBase.Width + radius;
            minYframe = -radius;
            maxYframe = rctBase.Height + radius;

            line = new Line()
            {
                X1 = xStart,
                Y1 = yStart,
                X2 = xEnd,
                Y2 = yEnd,
                StrokeThickness = 1,
                Stroke = new SolidColorBrush(Colors.Black)
            };
            cnsCan.Children.Add(line);

            Canvas.SetLeft(start, xStart - radius);
            Canvas.SetTop(start, yStart - radius);
            Canvas.SetLeft(end, xEnd - radius);
            Canvas.SetTop(end, yEnd - radius);

            LoadData();

            cbxSpread.Items.Add("Pad");
            cbxSpread.Items.Add("Reflect");
            cbxSpread.Items.Add("Repeat");
            cbxSpread.SelectedValue = "Pad";
        }

        /// <summary>
        /// Loading start coordinates of the point
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void LoadStartCoords(object sender, MouseButtonEventArgs e)
        {
            pointStart = (UIElement)sender;
            Mouse.Capture(pointStart);
            captured = true;
            xStart = Canvas.GetLeft(pointStart);
            x_canvasS = e.GetPosition(cnsCan).X;
            yStart = Canvas.GetTop(pointStart);
            y_canvasS = e.GetPosition(cnsCan).Y;
        }

        /// <summary>
        /// Start point shift
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void TranslateStart(object sender, MouseEventArgs e)
        {
            if (captured)
            {
                double x = e.GetPosition(cnsCan).X;
                double y = e.GetPosition(cnsCan).Y;

                if (x >= minXframe && x <= maxXframe)
                {
                    xStart += x - x_canvasS;
                    Canvas.SetLeft(pointStart, xStart);
                    x_canvasS = x;
                }

                if (y >= minYframe && y <= maxYframe)
                {
                    yStart += y - y_canvasS;
                    Canvas.SetTop(pointStart, yStart);
                    y_canvasS = y;
                }

                cnsCan.Children.Remove(line);
                line = new Line()
                {
                    X1 = xStart + radius,
                    Y1 = yStart + radius,
                    X2 = Canvas.GetLeft(end) + radius,
                    Y2 = Canvas.GetTop(end) + radius,
                    StrokeThickness = 1,
                    Stroke = new SolidColorBrush(Colors.Black)
                };

                cnsCan.Children.Add(line);

                LoadData();
            }
        }

        /// <summary>
        /// New start point placing
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void StopStart(object sender, MouseButtonEventArgs e)
        {
            Mouse.Capture(null);
            captured = false;
        }

        /// <summary>
        /// Loading end point coordinates
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void LoadEndCoords(object sender, MouseButtonEventArgs e)
        {
            pointEnd = (UIElement)sender;
            Mouse.Capture(pointEnd);
            captured = true;
            xEnd = Canvas.GetLeft(pointEnd);
            x_canvasE = e.GetPosition(cnsCan).X;
            yEnd = Canvas.GetTop(pointEnd);
            y_canvasE = e.GetPosition(cnsCan).Y;
        }

        /// <summary>
        /// End point shift
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void TranslateEnd(object sender, MouseEventArgs e)
        {
            if (captured)
            {
                double x = e.GetPosition(cnsCan).X;
                double y = e.GetPosition(cnsCan).Y;

                if (x >= minXframe && x <= maxXframe)
                {
                    xEnd += x - x_canvasE;
                    Canvas.SetLeft(pointEnd, xEnd);
                    x_canvasE = x;
                }

                if (y >= minYframe && y <= maxYframe)
                {
                    yEnd += y - y_canvasE;
                    Canvas.SetTop(pointEnd, yEnd);
                    y_canvasE = y;
                }

                cnsCan.Children.Remove(line);
                line = new Line()
                {
                    X1 = Canvas.GetLeft(start) + radius,
                    Y1 = Canvas.GetTop(start) + radius,
                    X2 = xEnd + radius,
                    Y2 = yEnd + radius,
                    StrokeThickness = 1,
                    Stroke = new SolidColorBrush(Colors.Black)
                };

                cnsCan.Children.Add(line);
                LoadData();
            }
        }

        /// <summary>
        /// New end point placing
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void StopEnd(object sender, MouseButtonEventArgs e)
        {
            Mouse.Capture(null);
            captured = false;
        }

        /// <summary>
        /// Setting up the SpreadMethod property
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void SpreadChange(object sender, SelectionChangedEventArgs e)
        {
            if (cbxSpread != null)
            {
                string selected = cbxSpread.SelectedItem.ToString();
                switch (selected)
                {
                    case "Pad":
                        linear.SpreadMethod = GradientSpreadMethod.Pad;
                        break;
                    case "Reflect":
                        linear.SpreadMethod = GradientSpreadMethod.Reflect;
                        break;
                    case "Repeat":
                        linear.SpreadMethod = GradientSpreadMethod.Repeat;
                        break;
                    default:
                        break;
                }
            }
        }


        /// <summary>
        /// Setting up StartPoint and EndPoint
        /// </summary>
        private void LoadData()
        {
            double xS = Math.Round(xStart / maxX, 2);
            if (xStart < 0)
            {
                xS = 0;
            }
            if (xStart > rctBase.Width)
            {
                xS = 1;
            }
            double yS = Math.Round(yStart / maxY, 2);
            if (yStart < 0)
            {
                yS = 0;
            }
            if (yStart > rctBase.Height)
            {
                yS = 1;
            }
            double xE = Math.Round(xEnd / maxX, 2);
            if (xEnd < 0)
            {
                xE = 0;
            }
            if (xEnd > rctBase.Width)
            {
                xE = 1;
            }
            double yE = Math.Round(yEnd / maxY, 2);
            if (yEnd < 0)
            {
                yE = 0;
            }
            if (yEnd > rctBase.Height)
            {
                yE = 1;
            }

            tbxXstart.Text = xS.ToString();
            tbxYstart.Text = yS.ToString();
            tbxXend.Text = xE.ToString();
            tbxYend.Text = yE.ToString();

            startPoint = new Point(xS, yS);
            endPoint = new Point(xE, yE);

            linear.StartPoint = startPoint;
            linear.EndPoint = endPoint;
        }
    }
}
